/*
 * Created on 09-Sep-2003
 *
 */
package pong;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;

/**
 * Classe que representa um Paddle do jogo.
 * Esta classe disponibiliza metodos para mover e redezenhar o Paddle.
 * <p> ATENO: No  necessrio criar uma instncia desta classe directamente. Isto  efectuado
 * automticamente pela API fornecida.</p>
 * @author nuno
 */
public class Paddle {

	protected static final int VERTICAL = 1; // eixo dos xx
	protected static final int HORIZONTAL = 2; // eixo dos yy
	
	/**
	 * Jogador com o Paddle da esquerda.
	 */
	public static final int LEFT = 1;
	
	/**
	 * Jogador com o Paddle da direita.
	 */
	public static final int RIGHT = 2;
	
	/**
	 * Jogador com o Paddle de cima.
	 */
	public static final int TOP = 3;
	
	/**
	 * Jogador com o Paddle de baixo.
	 */
	public static final int BOTTOM = 4;
	
	private static final int LAR = 50;
	private static final int ALT = 10;

	// vertical or horizontal
	private int move_type;
	// left or right if it is vertical; top or bottom if it is horizontal
	private int side;
	// current position of the paddle
	private int currentX, currentY, width, height;

	// bounds of the field
	int x_field;
	int y_field;
	int w_field;
	int h_field;

	private boolean active;

	/**
	 * Construtor da classe.
	 * @param type tipo de paddle (vertical ou horizontal)
	 * @param side Local onde se dever colocar o Paddle: esquerda, direita, cima ou baixo.
	 * @param c canvas que representa o campo.
	 * @throws IllegalArgumentException se algum valor dos parametros  invalido.
	 */
	public Paddle(int type, int side, Canvas c) {
		move_type = type;
		this.side = side;
		setActive(false);
		x_field = c.getX();
		y_field = c.getY();
		w_field = c.getWidth();
		h_field = c.getHeight();
		if(move_type==VERTICAL){
			if(side==LEFT){
				currentY = (h_field-x_field)/2 - (LAR/2);
				currentX = 10;								
			}
			else if(side==RIGHT){
				currentY = (h_field-x_field)/2 - (LAR/2);
				currentX = w_field-ALT-10;				
			}
			else throw new IllegalArgumentException("VERTICAL must be LEFT or RIGHT.");
			width = ALT;
			height = LAR;			
		}
		else if(move_type == HORIZONTAL){
			if(side==TOP){
				currentY = 10;
				currentX = (w_field/2) - (LAR/2);								
			}
			else if(side==BOTTOM){
				currentY = h_field-ALT-10;
				currentX = (w_field/2) - (LAR/2);				
			}
			else throw new IllegalArgumentException("HORIZONTAL must be TOP or BOTTOM.");
			width = LAR;
			height = ALT;
		}
		else throw new IllegalArgumentException("First arg must be HORIZONTAL or VERTICAL.");
	}

	/**
	 * @param active estado do Paddle (activo ou no).
	 */
	protected void setActive(boolean active) {
		this.active = active;
	}

	protected boolean isActive() {
		return active;
	}

	/**
	 * Move o Paddle para uma dada posio e redezenha-o no ecran.
	 * @param p nova posio.
	 * @param g grficos do componente onde o Paddle dever ser desenhado.
	 */
	public void move(Point p, Graphics g){
		clear(g);
		i_move(p.x,p.y);
		paint(g);
	}

	/**
	 * Devolve a componente xx da posio do Paddle.
	 * @return a componente xx da posio do Paddle.
	 */
	public int getX() {
		return currentX;
	}

	/**
	 * Devolve a componente yy da posio do Paddle.
	 * @return a componente yy da posio do Paddle.
	 */
	public int getY() {
		return currentY;
	}

	/**
	 *  Devolve a altura do Paddle.
	 * @return a altura do Paddle.
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * Devolve o comprimento do Paddle.
	 * @return o comprimento do Paddle.
	 */
	public int getWidth() {
		return width;
	}
	
	protected boolean isBallOnPaddle(BallState b) {
		int right = x_field + currentX + width;
		int left = currentX - x_field;
		int top = currentY - y_field;
		int bottom =  y_field + currentY + height;
		int xPosition = b.getXPosition();
		int yPosition = b.getYPosition();
		int xSpeed = b.getXSpeed();
		int ySpeed = b.getYSpeed();
		 
		if (side == LEFT && xSpeed < 0 && yPosition > top && yPosition < bottom) {
			if (xPosition + xSpeed < right) {
				b.setXSpeed(-xSpeed);
				return true;
			}
		} else if (side == RIGHT && xSpeed > 0 && yPosition > top && yPosition < bottom) {
			if (xPosition + xSpeed > left) {
				b.setXSpeed(-xSpeed);
				return true;
			}
		}
		if (side == TOP && ySpeed < 0 && xPosition > left && xPosition < right) {
			if (yPosition + ySpeed < bottom) {
				b.setYSpeed(-ySpeed);
				return true;
			}
		} else if (side == BOTTOM && ySpeed > 0 && xPosition > left && xPosition < right) {
			if (yPosition + ySpeed > top) {
				b.setYSpeed(-ySpeed);
				return true;
			}
		}
		return false;
	}

	/**
	 * 
	 */
	private void clear(Graphics g){
		g.clearRect(currentX,currentY,width,height);
	}
	
	/**
	 * 
	 */
	private void i_move(int x, int y){
		if(move_type == VERTICAL){
			if(y+LAR > h_field)
				currentY = h_field-LAR;
			else
				currentY = y;
		}
		else{
			if(x+LAR > w_field)
				currentX = w_field-LAR;
			else
				currentX = x;
		}
	}
	
	/**
	 * 
	 */
	protected void paint(Graphics g){
		Color oldColor = g.getColor();
		Color paintColor = null;
		if(active)
			paintColor = Color.DARK_GRAY;
		else
			paintColor = Color.RED;
		g.setColor(paintColor);
		g.fillRect(currentX,currentY,width,height);
		g.setColor(oldColor);
	}

}
